# -*- coding: utf-8 -*-

test_config = dict(
    ssh_host='127.0.0.1',
    ssh_pass='pass'
)

digitaldemiurge = dict(
    ssh_host='10.10.10.2',
    ssh_user='vagrant',
    ssh_key='/home/vitalis/.vagrant.d/insecure_private_key',
)
